use std::{default, fs};

use clap::{command, Parser, ValueEnum};
use semver::Version;
use toml_edit::{value, DocumentMut};

#[derive(Parser)]
#[command(name = "cargo")]
#[command(bin_name = "cargo")]
enum CargoCli {
	Semver(SemverArgs),
}

#[derive(Clone, Debug, PartialEq, Eq, ValueEnum)]
pub enum VerType {
	Major,
	Minor,
	Patch,
}

#[derive(Clone, Debug, Parser, PartialEq, Eq)]
#[command(version, about, long_about = None)]
pub struct SemverArgs {
	#[arg(long("bump-patch"), default_value_t = false)]
	pub bump_patch: bool,

	#[arg(long("bump-minor"), default_value_t = false)]
	pub bump_minor: bool,

	#[arg(long("bump-major"), default_value_t = false)]
	pub bump_major: bool,

	#[arg(long("get"), value_enum)]
	pub get: Option<VerType>,
}

fn main() {
	let CargoCli::Semver(args) = CargoCli::parse();

	let toml = fs::read_to_string("./Cargo.toml").unwrap();
	let mut doc = toml.parse::<DocumentMut>().expect("invalid doc");

	let semver = doc["package"]["version"].as_str().unwrap();
	let semver = semver.trim();
	let mut semver = Version::parse(semver).expect("invalid semver");

	if let Some(ver_type) = args.get {
		let ver = match ver_type {
			VerType::Major => semver.major,
			VerType::Minor => semver.minor,
			VerType::Patch => semver.patch,
		};

		print!("{ver}");
		std::process::exit(0);
	}

	if !args.bump_major && !args.bump_minor && !args.bump_patch {
		panic!("Specify which version to bump by using --bump-patch --bump-minor or --bump-major");
	}

	if args.bump_patch {
		semver.patch += 1;
	}

	if args.bump_minor {
		semver.minor += 1;
	}

	if args.bump_major {
		semver.major += 1;
	}

	println!("New semver: {}", semver);

	doc["package"]["version"] = value(semver.to_string());
	fs::write("./Cargo.toml", doc.to_string()).unwrap();
}
